#include <fcntl.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/ioctl.h>
#include <sys/select.h>
#include <sys/stat.h>
#include <sys/termios.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#include <errno.h>
#include <math.h>
#include <sys/wait.h>

#include <glib.h>

#include <pulse/pulseaudio.h>
#include <pulse/glib-mainloop.h>

static struct {
	GMainLoop		*mainloop;
	int			i;
} g;

static pa_sample_spec sample_spec = {
    .format = PA_SAMPLE_S16LE,
    .rate = 44100,
    .channels = 2
};

static void audio_playback_write_callback(pa_stream *s, size_t length, void *userdata) 	{
	printf("%s:%d\n", __FUNCTION__, __LINE__);
}

static void audio_record_read_callback(pa_stream *s, size_t length, void *userdata) 	{
	printf("%s:%d\n", __FUNCTION__, __LINE__);
}


void audio_playback_init(pa_context *c) {
	int		rc;
	pa_cvolume	cv;
	pa_stream	*stream=NULL;

	stream=pa_stream_new(c, "playback", &sample_spec, NULL);
	if (!stream) {
		fprintf(stderr, "pa_stream_new() failed: %s\n", pa_strerror(pa_context_errno(c)));
		exit(-1);
	}

	pa_cvolume_set(&cv, sample_spec.channels, PA_VOLUME_NORM);

	rc=pa_stream_connect_playback(stream, NULL, NULL, 0, &cv, NULL);
	if (rc<0) {
		fprintf(stderr, "pa_stream_connect_playback() failed: %s\n", pa_strerror(pa_context_errno(c)));
		exit(-1);
	}
	
	pa_stream_set_write_callback(stream, audio_playback_write_callback, NULL);
}

void audio_record_init(pa_context *c) {
	int		rc;
	pa_cvolume	cv;
	pa_stream	*stream=NULL;

	stream=pa_stream_new(c, "record", &sample_spec, NULL);
	if (!stream) {
		fprintf(stderr, "pa_stream_new() failed: %s\n", pa_strerror(pa_context_errno(c)));
		exit(-1);
	}

	pa_cvolume_set(&cv, sample_spec.channels, PA_VOLUME_NORM);

	rc=pa_stream_connect_record(stream, NULL, NULL, 0);
	if (rc<0) {
		fprintf(stderr, "pa_stream_connect_record() failed: %s\n", pa_strerror(pa_context_errno(c)));
		exit(-1);
	}

	pa_operation_unref(pa_stream_cork(stream, 1, NULL, NULL));
	
	pa_stream_set_read_callback(stream, audio_record_read_callback, NULL);
}

/* This is called whenever the context status changes */
static void context_state_callback(pa_context *c, void *userdata) {
	assert(c);

	switch (pa_context_get_state(c)) {
		case PA_CONTEXT_CONNECTING:
		case PA_CONTEXT_AUTHORIZING:
		case PA_CONTEXT_SETTING_NAME:
			break;

		case PA_CONTEXT_READY: 
			fprintf(stderr, "Connection established.\n");
			audio_playback_init(c);
			audio_record_init(c);
			break;
		case PA_CONTEXT_TERMINATED:
			exit(-1);
			break;

		case PA_CONTEXT_FAILED:
		default:
			fprintf(stderr, "Connection failure: %s\n", pa_strerror(pa_context_errno(c)));
			exit(-1);
			break;
	}
    	return;
}

#define APP_NAME	"triggerrec"

static void audio_create_context(pa_mainloop_api *paa) {
	static pa_context	*context=NULL;

	/* Create a new connection context */
	if (!(context = pa_context_new(paa, APP_NAME))) {
		fprintf(stderr, "pa_context_new() failed.\n");
		exit(-1);
	}

	pa_context_set_state_callback(context, context_state_callback, NULL);

	/* Connect the context */
	pa_context_connect(context, NULL, 0, NULL);
}

static void audio_start(void ) {
	pa_glib_mainloop	*paml;
	pa_mainloop_api		*paa;

	paml = pa_glib_mainloop_new(NULL);
	paa = pa_glib_mainloop_get_api(paml);

	audio_create_context(paa);
}

int main(int argc, char **argv) {
	g.mainloop=g_main_loop_new(NULL, FALSE);

	audio_start();

	g_main_loop_run(g.mainloop);

	exit(0);
}
